import { Client } from '@elastic/elasticsearch';
import Redis from 'ioredis';
import { DataSource } from 'typeorm';
import { CardTagEntity, FRIEND_TYPE, IMedia } from '../main';
export interface IUserProfileRedis {
    id: number;
    avatar: string;
    name: string;
}
export declare class RedisHelper {
    private readonly dataSource;
    private readonly redis;
    private readonly elasticIndex;
    private readonly elastic;
    private REDIS_KEY;
    private readonly SECONDS_FROM_HOUR;
    constructor(dataSource: DataSource, nodeEnv: string, redis: Redis, elasticIndex: {
        techres_customers: string;
    }, elastic: Client);
    checkFriendType(currentUserId: string, targetUserId: string): Promise<FRIEND_TYPE>;
    syncWaitingConfirmDbToRedis(syncUserId: string): Promise<void>;
    syncWaitingResponseDbToRedis(syncUserId: string): Promise<void>;
    findMutualFriend(currentUserId: string, targetUserId: string): Promise<string[]>;
    userIdBlockList(userId: string): Promise<string[]>;
    syncBlockUserDbToRedis(syncUserId: string): Promise<string[]>;
    userUnfollowNewsfeed(userId: string): Promise<string[]>;
    syncUserUnfollowNewsfeed(syncUserId: string): Promise<string[]>;
    cardTagList(cardTagIds: number[]): Promise<{
        [card_tag_id: string]: CardTagEntity;
    }>;
    syncCardTagDbToRedis(cardTagId: number): Promise<CardTagEntity>;
    postIdHiddenList(userId: string): Promise<string[]>;
    syncPostIdHiddenDbToRedis(syncUserId: string): Promise<string[]>;
    userProfileList(userIdList: string[]): Promise<{
        [x: string]: IUserProfileRedis;
    }>;
    syncUserDbToRedis(userIds: string[]): Promise<Record<string, IUserProfileRedis>>;
    friendList(userId: string): Promise<string[]>;
    syncFriendDbToRedis(syncUserId: string): Promise<string[]>;
    conversationDisableNotifyList(userId: string): Promise<string[]>;
    syncConversationDisableNotifyDbToRedis(userId: string): Promise<string[]>;
    conversationPinList(userId: string): Promise<string[]>;
    synConversationPinDbToRedis(userId: string): Promise<string[]>;
    conversationBackgroundList(userId: string, conversationIdList: string[]): Promise<{
        [conversation_id: string]: object;
    }>;
    conversationUserBackground(conversationId: string, userIds: string[]): Promise<Record<string, IMedia>>;
    synConversationBackgroundDbToRedis(userId: string, conversationId: string): Promise<IMedia>;
    usersOnlineStatus(userIds: string[]): Promise<{}>;
    conversationHiddenList(userId: string): Promise<string[]>;
    syncConversationHiddenDbToRedis(userId: string): Promise<string[]>;
}
