"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RedisHelper = void 0;
const elasticsearch_1 = require("@elastic/elasticsearch");
const common_1 = require("@nestjs/common");
const ioredis_1 = require("ioredis");
const typeorm_1 = require("typeorm");
const main_1 = require("../main");
let RedisHelper = class RedisHelper {
    constructor(dataSource, nodeEnv, redis, elasticIndex, elastic) {
        this.dataSource = dataSource;
        this.redis = redis;
        this.elasticIndex = elasticIndex;
        this.elastic = elastic;
        this.SECONDS_FROM_HOUR = 24 * 3600;
        this.REDIS_KEY = new main_1.RedisKey(nodeEnv);
    }
    async checkFriendType(currentUserId, targetUserId) {
        if (currentUserId == targetUserId)
            return main_1.FRIEND_TYPE.ITS_ME;
        const hasExist = await this.redis.exists([
            this.REDIS_KEY.USER_INFO(currentUserId, 'FRIEND'),
            this.REDIS_KEY.USER_INFO(currentUserId, 'WAITING_CONFIRM'),
            this.REDIS_KEY.USER_INFO(currentUserId, 'WAITING_RESPONSE'),
        ]);
        if (hasExist !== 3) {
            await this.syncFriendDbToRedis(currentUserId);
            await this.syncWaitingConfirmDbToRedis(currentUserId);
            await this.syncWaitingResponseDbToRedis(currentUserId);
        }
        if (await this.redis.sismember(this.REDIS_KEY.USER_INFO(currentUserId, 'FRIEND'), targetUserId))
            return main_1.FRIEND_TYPE.FRIEND;
        if (await this.redis.sismember(this.REDIS_KEY.USER_INFO(currentUserId, 'WAITING_CONFIRM'), targetUserId))
            return main_1.FRIEND_TYPE.WAITING_CONFIRM;
        if (await this.redis.sismember(this.REDIS_KEY.USER_INFO(currentUserId, 'WAITING_RESPONSE'), targetUserId))
            return main_1.FRIEND_TYPE.WAITING_RESPONSE;
        return main_1.FRIEND_TYPE.NOT_FRIEND;
    }
    async syncWaitingConfirmDbToRedis(syncUserId) {
        const userIdFriend = await this.dataSource
            .getRepository(main_1.FriendEntity)
            .find({
            where: { user_id: syncUserId, type: main_1.FRIEND_TYPE.WAITING_CONFIRM },
            order: {
                user_friend_id: 'ASC',
            },
        })
            .then((friend) => friend.reduce((result, item) => {
            result.push(item.user_friend_id);
            return result;
        }, ['0']));
        await this.redis.sadd(this.REDIS_KEY.USER_INFO(syncUserId, 'WAITING_CONFIRM'), userIdFriend);
        await this.redis.expire(this.REDIS_KEY.USER_INFO(syncUserId, 'WAITING_CONFIRM'), this.SECONDS_FROM_HOUR);
    }
    async syncWaitingResponseDbToRedis(syncUserId) {
        const userIdFriend = await this.dataSource
            .getRepository(main_1.FriendEntity)
            .find({
            where: {
                user_id: syncUserId,
                type: main_1.FRIEND_TYPE.WAITING_RESPONSE,
            },
            order: {
                user_friend_id: 'ASC',
            },
        })
            .then((friend) => friend.reduce((result, item) => {
            result.push(item.user_friend_id);
            return result;
        }, ['0']));
        await this.redis.sadd(this.REDIS_KEY.USER_INFO(syncUserId, 'WAITING_RESPONSE'), userIdFriend);
        await this.redis.expire(this.REDIS_KEY.USER_INFO(syncUserId, 'WAITING_RESPONSE'), this.SECONDS_FROM_HOUR);
    }
    async findMutualFriend(currentUserId, targetUserId) {
        if (currentUserId == targetUserId)
            return [];
        const hasExist = await this.redis.exists([
            this.REDIS_KEY.USER_INFO(currentUserId, 'FRIEND'),
            this.REDIS_KEY.USER_INFO(targetUserId, 'FRIEND'),
        ]);
        if (hasExist != 2) {
            await this.syncFriendDbToRedis(currentUserId);
            await this.syncFriendDbToRedis(targetUserId);
        }
        const listMutualFriend = await this.redis.sinter([
            this.REDIS_KEY.USER_INFO(currentUserId, 'FRIEND'),
            this.REDIS_KEY.USER_INFO(targetUserId, 'FRIEND'),
        ]);
        listMutualFriend.splice(0, 1);
        return listMutualFriend;
    }
    async userIdBlockList(userId) {
        const hasExist = await this.redis.exists(this.REDIS_KEY.USER_INFO(userId, 'BLOCK'));
        if (!hasExist)
            return await this.syncBlockUserDbToRedis(userId);
        const blockList = await this.redis.smembers(this.REDIS_KEY.USER_INFO(userId, 'BLOCK'));
        blockList.splice(0, 1);
        return blockList;
    }
    async syncBlockUserDbToRedis(syncUserId) {
        const userIdBlock = await this.dataSource
            .getRepository(main_1.BlockUserEntity)
            .find({
            select: ['user_id', 'block_user_id'],
            where: [
                {
                    user_id: syncUserId,
                },
                {
                    block_user_id: syncUserId,
                },
            ],
        })
            .then((data) => data.reduce((result, item) => {
            result.push(...[item.user_id, item.block_user_id].filter((item) => item != syncUserId));
            return result;
        }, ['0']));
        this.redis.sadd(this.REDIS_KEY.USER_INFO(syncUserId, 'BLOCK'), userIdBlock);
        this.redis.expire(this.REDIS_KEY.USER_INFO(syncUserId, 'BLOCK'), this.SECONDS_FROM_HOUR);
        userIdBlock.splice(0, 1);
        return userIdBlock;
    }
    async userUnfollowNewsfeed(userId) {
        const hasExist = await this.redis.exists(this.REDIS_KEY.USER_INFO(userId, 'UN_FOLLOW'));
        if (!hasExist)
            return await this.syncUserUnfollowNewsfeed(userId);
        const unfollowNewsfeedList = await this.redis.smembers(this.REDIS_KEY.USER_INFO(userId, 'UN_FOLLOW'));
        unfollowNewsfeedList.splice(0, 1);
        return unfollowNewsfeedList;
    }
    async syncUserUnfollowNewsfeed(syncUserId) {
        const userUnfollowNewsfeed = await this.dataSource
            .getRepository(main_1.UserUnfollowNewsfeedEntity)
            .find({
            select: ['unfollow_user_id'],
            where: {
                user_id: syncUserId,
            },
        })
            .then((data) => data.reduce((result, item) => {
            result.push(item.unfollow_user_id);
            return result;
        }, ['0']));
        this.redis.sadd(this.REDIS_KEY.USER_INFO(syncUserId, 'UN_FOLLOW'), userUnfollowNewsfeed);
        this.redis.expire(this.REDIS_KEY.USER_INFO(syncUserId, 'UN_FOLLOW'), this.SECONDS_FROM_HOUR);
        userUnfollowNewsfeed.splice(0, 1);
        return userUnfollowNewsfeed;
    }
    async cardTagList(cardTagIds) {
        const redisKeyCardTagList = cardTagIds.map((cardTagId) => this.REDIS_KEY.CARD_TAG(`${cardTagId}`));
        const cardTagList = await this.redis.mget(...redisKeyCardTagList);
        const result = {};
        await Promise.all(cardTagList.map(async (item, index) => {
            if (!item) {
                const cardTagSync = await this.syncCardTagDbToRedis(cardTagIds[index]);
                result[cardTagIds[index]] = cardTagSync;
            }
            else {
                const cardTag = JSON.parse(item);
                result[cardTag.id] = cardTag;
            }
        }));
        return result;
    }
    async syncCardTagDbToRedis(cardTagId) {
        const cardTag = await this.dataSource.getRepository(main_1.CardTagEntity).findOne({
            where: {
                id: String(cardTagId),
                is_deleted: main_1.BOOLEAN.FALSE,
            },
        });
        if (!cardTag)
            return;
        this.redis.set(this.REDIS_KEY.CARD_TAG(String(cardTagId)), JSON.stringify(cardTag), 'EX', this.SECONDS_FROM_HOUR);
        return cardTag;
    }
    async postIdHiddenList(userId) {
        const hasExist = await this.redis.exists(this.REDIS_KEY.USER_INFO(userId, 'HIDDEN_POST'));
        if (!hasExist)
            return await this.syncPostIdHiddenDbToRedis(userId);
        const postHidden = await this.redis.smembers(this.REDIS_KEY.USER_INFO(userId, 'HIDDEN_POST'));
        postHidden.splice(0, 1);
        return postHidden;
    }
    async syncPostIdHiddenDbToRedis(syncUserId) {
        const postIdHiddenList = await this.dataSource
            .getRepository(main_1.UserReportEntity)
            .createQueryBuilder('ur')
            .select('r.content_id')
            .leftJoinAndSelect(main_1.ReportEntity, 'r', 'ur.report_id = r.id')
            .where('ur.user_id = :userId', {
            userId: syncUserId,
        })
            .andWhere('r.type = :type', {
            type: main_1.REPORT_TYPE.POST,
        })
            .getRawMany()
            .then((data) => data.reduce((result, item) => {
            result.push(item.content_id);
            return result;
        }, ['0']));
        this.redis.sadd(this.REDIS_KEY.USER_INFO(syncUserId, 'HIDDEN_POST'), postIdHiddenList);
        this.redis.expire(this.REDIS_KEY.USER_INFO(syncUserId, 'HIDDEN_POST'), this.SECONDS_FROM_HOUR);
        postIdHiddenList.splice(0, 1);
        return postIdHiddenList;
    }
    async userProfileList(userIdList) {
        const redisKeyUserList = userIdList.map((userId) => this.REDIS_KEY.USER_INFO(userId, 'PROFILE'));
        const userList = await this.redis.mget(...redisKeyUserList);
        const userIdSyncRedis = [];
        const usersRedis = userList.reduce((accumulator, currentValue, currentIndex) => {
            const user = JSON.parse(currentValue);
            const userId = userIdList[currentIndex];
            if (!user) {
                userIdSyncRedis.push(userId);
                accumulator[userId] = {};
            }
            else {
                accumulator[userId] = user;
            }
            return accumulator;
        }, {});
        const usersSyncRedis = userIdSyncRedis.length ? await this.syncUserDbToRedis(userIdSyncRedis) : {};
        return Object.assign(Object.assign({}, usersRedis), usersSyncRedis);
    }
    async syncUserDbToRedis(userIds) {
        const redisMulti = this.redis.multi();
        const users = await this.elastic
            .search({
            index: this.elasticIndex.techres_customers,
            _source: ['id', 'avatar', 'name'],
            query: {
                bool: {
                    must: [
                        {
                            terms: {
                                id: userIds,
                            },
                        },
                    ],
                },
            },
        })
            .then((result) => result.hits.hits.reduce((accumulator, currentValue) => {
            const user = currentValue._source;
            accumulator[user.id] = user;
            redisMulti.set(this.REDIS_KEY.USER_INFO(user.id, 'PROFILE'), JSON.stringify(user), 'EX', this.SECONDS_FROM_HOUR);
            return accumulator;
        }, {}));
        await redisMulti.exec();
        return users;
    }
    async friendList(userId) {
        const friendList = await this.redis.smembers(this.REDIS_KEY.USER_INFO(userId, 'FRIEND'));
        if (friendList.length === 0)
            return await this.syncFriendDbToRedis(userId);
        friendList.splice(0, 1);
        return friendList;
    }
    async syncFriendDbToRedis(syncUserId) {
        const userIdFriend = await this.dataSource
            .getRepository(main_1.FriendEntity)
            .find({
            where: {
                user_id: syncUserId,
                type: main_1.FRIEND_TYPE.FRIEND,
            },
            order: {
                user_friend_id: 'ASC',
            },
        })
            .then((friend) => friend.reduce((result, item) => {
            result.push(item.user_friend_id);
            return result;
        }, ['0']));
        await this.redis.sadd(this.REDIS_KEY.USER_INFO(syncUserId, 'FRIEND'), userIdFriend);
        await this.redis.expire(this.REDIS_KEY.USER_INFO(syncUserId, 'FRIEND'), this.SECONDS_FROM_HOUR);
        userIdFriend.splice(0, 1);
        return userIdFriend;
    }
    async conversationDisableNotifyList(userId) {
        const conversationDisableNotify = await this.redis.smembers(this.REDIS_KEY.USER_INFO(userId, 'CONVERSATION_DISABLE_NOTIFY'));
        if (conversationDisableNotify.length === 0)
            return await this.syncConversationDisableNotifyDbToRedis(userId);
        conversationDisableNotify.splice(0, 1);
        return conversationDisableNotify;
    }
    async syncConversationDisableNotifyDbToRedis(userId) {
        const conversationDisableNotify = await this.dataSource
            .getRepository(main_1.ConversationUserDisableNotifyEntity)
            .find({
            select: ['conversation_id'],
            where: {
                user_id: userId,
            },
        })
            .then((data) => data.reduce((result, item) => {
            result.push(item.conversation_id);
            return result;
        }, ['0']));
        this.redis.sadd(this.REDIS_KEY.USER_INFO(userId, 'CONVERSATION_DISABLE_NOTIFY'), conversationDisableNotify);
        this.redis.expire(this.REDIS_KEY.USER_INFO(userId, 'CONVERSATION_DISABLE_NOTIFY'), this.SECONDS_FROM_HOUR);
        conversationDisableNotify.splice(0, 1);
        return conversationDisableNotify;
    }
    async conversationPinList(userId) {
        const pinnedConversations = await this.redis
            .get(this.REDIS_KEY.USER_INFO(userId, 'CONVERSATION_PINNED'))
            .then((result) => JSON.parse(result));
        if (!pinnedConversations)
            return await this.synConversationPinDbToRedis(userId);
        return pinnedConversations;
    }
    async synConversationPinDbToRedis(userId) {
        const conversationPin = await this.dataSource
            .getRepository(main_1.ConversationUserPinnedEntity)
            .find({
            select: ['id', 'conversation_id'],
            where: {
                user_id: userId,
            },
            order: {
                id: 'DESC',
            },
        })
            .then((result) => result.map((item) => item.conversation_id));
        await this.redis.set(this.REDIS_KEY.USER_INFO(userId, 'CONVERSATION_PINNED'), JSON.stringify(conversationPin), 'EX', this.SECONDS_FROM_HOUR);
        return conversationPin;
    }
    async conversationBackgroundList(userId, conversationIdList) {
        const redisKeyConversationBackgroundList = conversationIdList.map((conversationId) => this.REDIS_KEY.CONVERSATION_USER_BACKGROUND(userId, conversationId));
        const conversationBackgroundList = await this.redis.mget(...redisKeyConversationBackgroundList);
        const result = {};
        await Promise.all(conversationBackgroundList.map(async (item, index) => {
            const conversationBackground = JSON.parse(item);
            if (!conversationBackground) {
                result[conversationIdList[index]] = await this.synConversationBackgroundDbToRedis(userId, conversationIdList[index]);
            }
            else {
                result[conversationIdList[index]] = conversationBackground;
            }
        }));
        return result;
    }
    async conversationUserBackground(conversationId, userIds) {
        const redisKeys = userIds.map((userId) => this.REDIS_KEY.CONVERSATION_USER_BACKGROUND(userId, conversationId));
        const conversationUserBackgrounds = await this.redis.mget(...redisKeys);
        const result = {};
        for (let i = 0; i < conversationUserBackgrounds.length; i++) {
            const element = JSON.parse(conversationUserBackgrounds[i]);
            if (!element) {
                result[userIds[i]] = await this.synConversationBackgroundDbToRedis(conversationId, userIds[i]);
            }
            else {
                result[userIds[i]] = element;
            }
        }
        return result;
    }
    async synConversationBackgroundDbToRedis(userId, conversationId) {
        const conversationBackground = await this.dataSource.getRepository(main_1.ConversationUserBackgroundEntity).findOne({
            select: ['background'],
            where: {
                user_id: userId,
                conversation_id: conversationId,
            },
        });
        if (!conversationBackground)
            return;
        this.redis.set(this.REDIS_KEY.CONVERSATION_USER_BACKGROUND(userId, conversationId), JSON.stringify(conversationBackground.background), 'EX', this.SECONDS_FROM_HOUR);
        return conversationBackground.background;
    }
    async usersOnlineStatus(userIds) {
        const keysMulti = this.redis.multi();
        for (const userId of userIds) {
            keysMulti.keys(this.REDIS_KEY.USER_CONNECTION(userId, '*'));
        }
        return await keysMulti.exec().then((data) => data.reduce((accumulator, [error, result], currentIndex) => {
            accumulator[userIds[currentIndex]] = result.length ? main_1.BOOLEAN.TRUE : main_1.BOOLEAN.FALSE;
            return accumulator;
        }, {}));
    }
    async conversationHiddenList(userId) {
        const conversationHiddenIds = await this.redis
            .get(this.REDIS_KEY.USER_INFO(userId, 'CONVERSATION_HIDDEN'))
            .then((result) => JSON.parse(result));
        if (!conversationHiddenIds)
            return await this.syncConversationHiddenDbToRedis(userId);
        return conversationHiddenIds;
    }
    async syncConversationHiddenDbToRedis(userId) {
        const conversationHidden = await this.dataSource
            .getRepository(main_1.ConversationUserHiddenEntity)
            .find({
            select: ['conversation_id'],
            where: {
                user_id: userId,
            },
        })
            .then((result) => result.map((item) => item.conversation_id));
        await this.redis.set(this.REDIS_KEY.USER_INFO(userId, 'CONVERSATION_HIDDEN'), JSON.stringify(conversationHidden), 'EX', this.SECONDS_FROM_HOUR);
        return conversationHidden;
    }
};
exports.RedisHelper = RedisHelper;
exports.RedisHelper = RedisHelper = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [typeorm_1.DataSource, String, ioredis_1.default, Object, elasticsearch_1.Client])
], RedisHelper);
//# sourceMappingURL=redis-helper.service.js.map