import { BeforeInsert, Column, Entity } from 'typeorm';
import { BOOLEAN, IMedia, ISticker, ITag, IThumbnail, IUser } from '../enums/_common.enum';
import { IMessageUserTarget, MESSAGE_STATUS, MESSAGE_TYPE } from '../enums/message.enum';
import { normalizedString } from '../utils/normalized-string';
import { CommonEntity } from './_common.entity';

@Entity('message')
export class MessageEntity extends CommonEntity {
  @Column({
    type: 'bigint',
  })
  conversation_id: string;

  @Column({
    type: 'bigint',
  })
  user_id: string;

  @Column({
    type: 'json',
    nullable: true,
  })
  user_target: IMessageUserTarget[];

  @Column({
    type: 'text',
    nullable: true,
  })
  message: string;

  @Column({
    type: 'json',
    nullable: true,
  })
  media: IMedia[];

  @Column({
    type: 'json',
    nullable: true,
  })
  thumb: IThumbnail;

  @Column({
    type: 'json',
    nullable: true,
  })
  sticker: ISticker;

  @Column({
    type: 'json',
    nullable: true,
  })
  tag: ITag[];

  @Column({
    type: 'int',
    default: 0,
  })
  no_of_reaction: number;

  @Column({
    type: 'smallint',
    default: BOOLEAN.FALSE,
  })
  is_pinned: BOOLEAN;

  @Column({
    type: 'smallint',
    default: BOOLEAN.FALSE,
  })
  is_share: BOOLEAN;

  @Column({
    type: 'smallint',
    default: BOOLEAN.FALSE,
  })
  is_timeline: BOOLEAN;

  @Column({
    type: 'bigint',
    nullable: true,
  })
  message_reply_id: string;

  @Column({
    type: 'int',
    default: 0,
  })
  no_of_like: number;

  @Column({
    type: 'int',
    default: 0,
  })
  no_of_love: number;

  @Column({
    type: 'int',
    default: 0,
  })
  no_of_wow: number;

  @Column({
    type: 'int',
    default: 0,
  })
  no_of_sad: number;

  @Column({
    type: 'int',
    default: 0,
  })
  no_of_haha: number;

  @Column({
    type: 'int',
    default: 0,
  })
  no_of_angry: number;

  @Column({
    type: 'json',
    default: [],
  })
  viewed_users: IUser[];

  @Column({
    type: 'varchar',
    nullable: true,
  })
  link_join_group: string;

  @Column({
    type: 'bigint',
    nullable: true,
  })
  message_vote_id: string;

  @Column({
    type: 'smallint',
  })
  type: MESSAGE_TYPE;

  @Column({
    type: 'smallint',
    default: MESSAGE_STATUS.ACTIVE,
  })
  status: MESSAGE_STATUS;

  @Column({
    type: 'text',
  })
  search_value_normalized: string;

  @BeforeInsert()
  beforeInsert() {
    this.searchValueNormalized();
  }

  private searchValueNormalized() {
    this.search_value_normalized = normalizedString(this.message || '');
  }

  conversation_updated_at: number;
}
