import { Client } from '@elastic/elasticsearch';
import { DynamicModule, InjectionToken, Module, OptionalFactoryDependency } from '@nestjs/common';
import { getDataSourceToken } from '@nestjs/typeorm';
import Redis from 'ioredis';
import { DataSource } from 'typeorm';
import { RedisHelper } from './redis-helper.service';

@Module({})
export class RedisHelperModule {
  static register(options: {
    ioRedis: InjectionToken | OptionalFactoryDependency;
    elastic: InjectionToken | OptionalFactoryDependency;
    nodeEnv: string;
    elastic_index: {
      techres_customers: string;
    };
    dataSourceName?: string;
  }): DynamicModule {
    const { ioRedis, elastic, nodeEnv, elastic_index, dataSourceName } = options;

    return {
      global: true,
      module: RedisHelperModule,
      providers: [
        {
          provide: RedisHelper,
          useFactory: (connection: DataSource, redis: Redis, elastic: Client) => {
            return new RedisHelper(connection, nodeEnv, redis, elastic_index, elastic);
          },
          inject: [getDataSourceToken(dataSourceName), ioRedis, elastic],
        },
      ],
      exports: [RedisHelper],
    };
  }
}
